<?php

namespace ID3\Bases;

class Plugin{

    public $rest                = null;
    public $shortcode           = null;

    public $_namespace          = '';
    public $name                = '';
    public $description         = '';
    public $version             = '';
    public $root                = '';
    public $dir                 = '';
    public $url                 = '';
    public $key                 = '';
    public $tdom                = '';


    public $settings            = null;
    public $networkSettings     = null;
    public $scripts             = null;
    public $styles              = null;
    public $actions             = [];
    public $filters             = [];
    public $modules             = [];

    private $_body_classes      = [];
    private $_text_hashtable    = null;
    private $_loaded_text_hash  = [];


    public $logHandler          = null;

    function __construct($namespace){
        $this->_namespace          = $namespace;
        $this->name                = constant($namespace . '\NAME');
        $this->description         = constant($namespace . '\DESC');
        $this->version             = constant($namespace . '\VERSION');
        $this->root                = constant($namespace . '\PLUGIN');
        $this->dir                 = constant($namespace . '\PLUGIN_DIR');
        $this->url                 = constant($namespace . '\PLUGIN_URL');
        $this->key                 = constant($namespace . '\KEY');
        $this->color               = constant($namespace . '\COLOR');
        $this->tdom                = constant($namespace . '\TDOM');

        $this->logHandler           = new PluginLogger($this->key);
    }

    public function init(){
        $this->addBodyClass($this->key);

        $this->loadModules($this->_namespace);
        $this->registerRest($this->_namespace);
        $this->registerBlocks($this->_namespace);
        $this->registerShortcodes($this->_namespace);

        $this->registerActions();
        $this->registerFilters();


    }


    #region MODULES

    public function loadModules($namespace){
        
        $the_plugs = get_option('active_plugins'); 
        if(is_multisite()){
            $the_network_plugs = get_site_option('active_sitewide_plugins');
            foreach ($the_network_plugs as $key => $value) {
                $the_plugs[] = $key;  
            }
        }
    
        foreach($the_plugs as $key => $value) {

            $moduleName = strtolower(explode("/", $value)[0]);
            
            $modulePath = $this->dir . "modules/$moduleName/functions.php";
            if(file_exists($modulePath)){
                include $modulePath;
            }
        }

        if($this->has_modules()){
            foreach($this->modules as $module){
                $module->init($namespace);

                $module->includes();
                $module->register_actions();
                $module->register_filters();
                $module->register_locales();
            }
        }
    }

    public function has_modules(){
        return $this->modules != null && count($this->modules)>0;
    }

    public function registerModule($moduleInstance){
        if($this->modules == null){
            $this->modules = [];
        }
        $this->modules[] = $moduleInstance;
    }

    #endregion MODULES
    
    public function loadSettings(){
        if($this->settings != null) return;
        
        $this->settings = \ID3\Settings::get($this->key);
        if($this->settings == null){
            $this->settings = \ID3\Settings::getNetwork($this->key);
        }
        else{
            $this->networkSettings = \ID3\Settings::getNetwork($this->key);
        }
        return;

    }

    public function saveSettings(){
        \ID3\Settings::set($this->key, $this->settings);
        return;
    }

    public function addBodyClass($class){
        $this->_body_classes[] = $class;
    }
    public function applyBodyClass($classes=[]){
        $classes = array_merge($classes, $this->_body_classes);
        return $classes;
    }
    public function addAction($tag, $fnName, $priority = 5){
        $this->actions[] = [
            'tag' => $tag,
            'func' => $fnName,
            'priority' => $priority
        ];
    }
    public function addFilter($tag, $fnName, $priority = 5){
        $this->filters[] = [
            'tag' => $tag,
            'func' => $fnName,
            'priority' => $priority
        ];
    }

    public function addStyles($styles){
        foreach ($styles as $item){
            $this->styles[] = $item;
        }
    }
    public function registerStyles(){
        if($this->styles == null) return;

        foreach ($this->styles as $item) {
            wp_enqueue_style($item['key'], $item['path']);
        }
    }

    public function addScripts($script){
        foreach ($script as $item){
            $this->scripts[] = $item;
        }
    }
    public function registerScripts(){
        if(method_exists($this, 'renderScripts')){
            //$this->renderScripts;
            return;
        }
        if($this->scripts == null) return;

        
        foreach ($this->scripts as $item) {
            wp_enqueue_script($item['key'], $item['path'],[],false,true);
        }
    }

    public function registerActions(){

        // basic actions
        
        // localized actions
        foreach ($this->actions as $item) {  
            
            if(method_exists($this, $item['func'])){
                $fct = new \ReflectionMethod($this, $item['func']);
                //$paramCount = $fct->getNumberOfRequiredParameters();
                add_action($item['tag'], [$this, $item['func']], $item['priority']);
            }
        }
    }

    public function registerBlocks(){
        if(file_exists($this->dir . 'core/class.block.php')){
            require_once($this->dir . 'core/class.block.php');
            // $rest_cname = $namespace . '\\block';
            // $this->block = new $rest_cname;
        }
    }

    public function registerFilters(){
        // basic filters
        add_filter( 'gettext', array($this, 'translate'), 5, 3 );
        add_filter( 'body_class', [$this, 'applyBodyClass'], 5, 3 );

        // localized filters
        foreach ($this->filters as $item) {    
            if(method_exists($this, $item['func'])){
                $fct = new \ReflectionMethod($this, $item['func']);
                $paramCount = $fct->getNumberOfRequiredParameters();
                add_filter($item['tag'], [$this, $item['func']], $item['priority'], $paramCount);
            }
        }
    }

    public function registerShortcodes($namespace){
        if(file_exists($this->dir . 'core/class.shortcode.php')){
            require_once($this->dir . 'core/class.shortcode.php');
            $shortcode_cname = $namespace . '\\shortcode';
            $this->shortcodes = new $shortcode_cname;
        }
    }

    public function registerRest($namespace){
        if(file_exists($this->dir . 'core/class.rest.php')){
            require_once($this->dir . 'core/class.rest.php');
            $rest_cname = $namespace . '\\rest';
            $this->rest = new $rest_cname;
        }
    }


    public function translate($translated_text, $text, $domain){
        if($translated_text != $text) return $translated_text;
        if(in_array($domain,[$this->tdom,'id3-frwk'])){
            $textHashTable = $this->getTranslationHashTable();

            if(isset($textHashTable[$text])){
                $translated_text = $textHashTable[$text];
            }
        }

        return $translated_text;
    }

    /**
     * Return the translation hashtable
     * Load the hashtable from global framework translations on the first access
     * Append plugin translation to hashtable if not loaded
     * @return array Associative array of key => translation
     */
    public function getTranslationHashTable(){
        if($this->_text_hashtable != null) {
            if(in_array($this->tdom,$this->_loaded_text_hash)){
                return $this->_text_hashtable;
            }
        }

        $lang = substr(get_locale(),0,2);
        $globalText = []; 
        $pluginText = [];

        if($this->_text_hashtable != null) {
            $globalText = $this->_text_hashtable;
        }
        else{
            $globalText = $this->_loadTranslationFile(\ID3\Framework\PLUGIN_DIR . "/scripts/locales/{$lang}.json");
            $this->_loaded_text_hash[] = 'id3-frwk';
        }

        if($this->tdom != 'id3-frwk'){
            $pluginText = $this->_loadTranslationFile($this->dir . "/scripts/locales/{$lang}.json");
            $this->_loaded_text_hash[] = $this->tdom;
        }

        $this->_text_hashtable = array_merge($globalText,$pluginText);
        return $this->_text_hashtable;
    }

    /**
     * Load translation file
     * @param string $path file path to .json file
     * @return array Associative array of key => translation, empty array if file does not exists
     */
    private function _loadTranslationFile(String $path){
        if(!file_exists($path)) return [];

        return json_decode(file_get_contents($path),true);
    }
    
    public function log($message, $includeRequestMeta=true){

        $this->logHandler->write($message, $includeRequestMeta);
        
    }
}

class PluginLogger{

    public $path = '';

    function __construct($name){
        $lUploadDir   = wp_upload_dir();
        $this->path = $lUploadDir['basedir'] . '/_' . sanitize_title($name) . '.log';
    }

    function write($message, $includeRequestMeta=true){
        if(defined('ID3\ENABLE_LOG') && !\ID3\FRAMEWORK\ENABLE_LOG ) return;

        $filePointer = fopen($this->path,'a');
        $timestamp = Date('Y-M-d H:i:s');

        if(is_array($message) || is_object($message)){
            $message = json_encode($message);
        }
        if($includeRequestMeta){
            $referer = $_SERVER['HTTP_REFERER'] ?: 'None';
            $request = $_SERVER['REQUEST_URI'];
            $method  = $_SERVER['REQUEST_METHOD'];
            $message = "$message ($method : $request, referer:'$referer') ";
        }

        fwrite($filePointer, "$timestamp :: $message \n");
        fclose($filePointer);
    }

}