<?php
namespace ID3\Framework;

use ID3\Bases\Plugin;

use function ID3\debug;

class Main extends \ID3\Bases\MainInstance{

    function __construct(){

        parent::__construct(__NAMESPACE__);

        add_action('wp_head',[$this,'head']);
        add_action('language_attr', [$this, 'setHtmlAttributes']);

        $this->addScript('https://kit.fontawesome.com/53b266840c.js');
    }

    
    public function setHtmlAttributes($attr){
        $version = VERSION;
        return "{$attr} id3-framework=\"{$version}\" ";
    }
    
    function head(){
        $this->loadSettings();

        if(isset($this->settings->color_palette)){
            $palette = new ColorPaletteBuilder($this->settings->color_palette);
            $palette->generateStyle();
        }
        
        if(isset($this->settings->style_vars)){
            $styles = new UniversalStyleBuilder($this->settings->style_vars);
            $styles->generateStyle();
        }

        echo('<script type="text/javascript">');
        echo('if(id3 == undefined){var id3 = {settings:{}};};');
        echo('id3.settings["id3-framework"]=' . json_encode($this->settings) . ';');
        echo('</script>');
    }


    function registerAdminScripts(){
        wp_enqueue_script(
            'id3fw-prototype-js',
            '/' . $this->url . 'scripts/public/_prototypes.js',
        );

        wp_enqueue_script(
            'id3fw-fa-js',
            'https://kit.fontawesome.com/53b266840c.js'
        );
        wp_enqueue_script(
            'id3fw-blocks-js',
            '/' . $this->url . 'scripts/fw-blocks.min.js',
            [ 'id3fw-prototype-js', 'wp-i18n', 'wp-element', 'wp-blocks', 'wp-components', 'wp-editor'],
            filemtime( $this->dir . 'scripts/fw-blocks.min.js' )
        );
    }

}

new Main();


class ColorPaletteBuilder{

    public $palette;
    public $root;

    function __construct($palette, $root='body'){
        $this->palette = $palette;
        $this->root = $root;    
    }

    function generateStyle(){
        
        echo('<style id="id3-fw-color-palette">');
        echo($this->root . '{');
        echo($this->buildColorPalettesDefinition());
        echo('}');
        echo('</style>');
    }


    function buildColorPalettesDefinition(){

        $palettes = [];
        if(isset($this->palette)){
            $palettes = array_merge($palettes, $this->buildColorPalette('primary'));
            $palettes = array_merge($palettes, $this->buildColorPalette('secondary'));
            $palettes = array_merge($palettes, $this->buildColorPalette('action'));

        }

        // Allow palette filtering/hook
        apply_filters('id3-fw/color-palette-builder/styles', $palettes);

        return implode(';',$palettes);
    }
    function buildColorPalette($paletteName){
        if(!isset($this->palette->{$paletteName})) return [];
        $vars = [];
        $palette = $this->palette->{$paletteName};

        $vars[] = "--id3-color-{$paletteName}: hsl(" . $this->getPaletteParts($palette) . ')';
        $vars[] = "--id3-color-{$paletteName}-revert: hsl(" . $this->getPaletteParts($palette,'hs') . ',' . ($palette->l >= 50 ? 0 : 100) .'%)';

        $refPoint = $palette->l;

        for($i=10; $i<=90; $i+=10){            
            $delta = $refPoint >= 50 ? -1 * $i : $i;
            $lightness  = $refPoint + ($refPoint / 100 * $delta);
            $alpha      = $i / 100;
            $vars[] = "--id3-color-{$paletteName}-l-{$i}:" . $lightness . '%';
            $vars[] = "--id3-color-{$paletteName}-a{$i}: hsla(" . $this->getPaletteParts($palette) . ',' . $alpha . ')';
        }

        // contrast
        $contrastMultiplier = ($palette->l >= 50) ? -1 : 1;
        $contrastSpace = ($palette->l >= 50) ? $palette->l : 100 - $palette->l;

        $vars[] = "--id3-color-{$paletteName}-contrast-min: hsl(" .  $this->getPaletteParts($palette,'hs') . ',' . ($palette->l + ($contrastSpace * 0.1 * $contrastMultiplier)) . '%)';
        $vars[] = "--id3-color-{$paletteName}-contrast-mid: hsl(" .  $this->getPaletteParts($palette,'hs') . ',' . ($palette->l + ($contrastSpace * 0.2 * $contrastMultiplier)) . '%)';
        $vars[] = "--id3-color-{$paletteName}-contrast-max: hsl(" .  $this->getPaletteParts($palette,'hs') . ',' . ($palette->l + ($contrastSpace * 0.3 * $contrastMultiplier)) . '%)';


        $vars[] = "--id3-color-{$paletteName}-l-100:" . ($refPoint >= 50 ? 0 : 100) . '%';
        $vars[] = "--id3-color-base-{$paletteName}:" . $this->getPaletteParts($palette,'hs');

        return $vars;
    }

    function getPaletteParts($palette=null, $parts='hsl'){
        if($palette == null) return '0,0,0';
        $result = [];
        if(strpos($parts,'h') !== false) $result[] = $palette->h;
        if(strpos($parts,'s') !== false) $result[] = $palette->s . '%';
        if(strpos($parts,'l') !== false) $result[] = $palette->l . '%';

        return implode(',', $result);
    }

}

class UniversalStyleBuilder{
    public $styleVars;
    public $root;

    const PROP_BASE = '--id3-ui';

    function __construct($styleVars, $root='body'){
        $this->styleVars = $styleVars;
        $this->root = $root;    
    }

    function generateStyle(){
        
        echo('<style id="id3-fw-style-vars">');
        echo($this->root . '{');
        echo($this->buildStyleVarsDefinition());
        echo('} ');

        echo($this->buildStyleVarsClasses());

        echo('</style>');
    }

    function buildStyleVarsDefinition(){
        if(!isset($this->styleVars)) return '';

        $result = [];
        
        foreach($this->styleVars as $key => $var){
            if(strpos($key, '_button') !== false) continue;
            $base = self::PROP_BASE;
            $result[] = "{$base}-{$key}:{$this->getUnitBaseValue($var)}";
        }

        return implode(';', $result);
    }

    function getUnitBaseValue($var){
        if(is_object($var)){
            return "{$var->value}{$var->unit}";
        }
        return $var;
        
    }

    function buildStyleVarsClasses(){
        if(!isset($this->styleVars)) return '';

        $result = [];
        if(isset($this->styleVars->normal_button)) $result[] = $this->buildButtonClass('normal', $this->styleVars->normal_button);
        if(isset($this->styleVars->action_button)) $result[] = $this->buildButtonClass('action', $this->styleVars->action_button);

        return implode("\n ", $result);
    }

    function buildButtonClass($type, $options){
        
        $colorType = ($type == 'action') ? 'action' : 'secondary';
        $result = [
            // "--id3-ui-button-color: var(--id3-color-{$colorType})",
            // "--id3-ui-button-color-base: var(--id3-color-base-{$colorType})",
            // "--id3-ui-button-color-contrast-min: var(--id3-color-{$colorType}-contrast-min)",
            // "--id3-ui-button-color-contrast-mid: var(--id3-color-{$colorType}-contrast-mid)",
            // "--id3-ui-button-color-contrast-max: var(--id3-color-{$colorType}-contrast-max)",
            // "--id3-ui-button-color-revert: var(--id3-color-{$colorType}-revert)",
        ];

        $styleMap     = [
            // 'hollow' => implode(';',[
            //     "--id3-ui-button-bg: transparent",
            //     "--id3-ui-button-fg: var(--id3-color-{$colorType})",
            //     "--id3-ui-button-bg-hover: var(--id3-color-{$colorType})",
            //     "--id3-ui-button-fg-hover: var(--id3-color-{$colorType}-revert)",
            //     "--id3-ui-button-border: solid 2px var(--id3-color-{$colorType})"
            // ]),
            // 'fill' => implode(';', [
            //     "--id3-ui-button-bg: var(--id3-color-{$colorType})",
            //     "--id3-ui-button-fg: var(--id3-color-{$colorType}-revert)",
            //     "--id3-ui-button-bg-hover: var(--id3-color-{$colorType}-revert)",
            //     "--id3-ui-button-fg-hover: var(--id3-color-{$colorType})",
            //     "--id3-ui-button-border: none"
            // ])
        ];
        $roundnessMap = [
            'max' => '--id3-ui-corner: 2em',
            'none' => '--id3-ui-corner: 0'
        ];

        //$result[] = $styleMap[$options->style];
        if(isset($roundnessMap[$options->roundness])) $result[] = $roundnessMap[$options->roundness];
        // $result[] = "--id3-ui-hover-animation: id3-anim-{$options->hover_animation}";
        // $result[] = "--id3-ui-revert-animation: id3-anim-{$options->hover_animation}-revert";

        return ".id3-button.id3-type-{$type}{" .
                    implode(';',$result) .
                "}";
    }
}

class Components extends \ID3\Bases\Plugin{

    function __construct(){
        parent::__construct(__NAMESPACE__);
        $this->loadSettings();
    }

    function Button($content='', $options=[]){
        
        $options = array_merge([
            'tag'           => 'div',
            'href'          => null,
            'class'         => 'id3-type-normal',
            'style'         => 'id3-style-' . $this->settings->style_vars->normal_button->style,
            'text_transform'=> 'id3-text-' . $this->settings->style_vars->normal_button->text_transform,
            'hover_effect'  => 'id3-trans-' . $this->settings->style_vars->normal_button->hover_animation,
            'icon_display'  => 'id3-icon-' . $this->settings->style_vars->normal_button->icon_display,
            'icon'          => $this->settings->style_vars->normal_button->icon,
        ],$options);

        extract($options);

        $iconTag = '';

        $classes = ['id3-button', $class, $text_transform, $hover_effect, $style];
        if($icon_display != 'id3-icon-none' && $icon != '') {
            $classes[] = $icon_display;
            $iconTag = "<i class=\"id3-icon {$icon}\"></i>";
        }

        $classes = implode(' ', $classes);
        
        $tag = ($href != null) ? 'a' : $tag;
        $attrs = [];
        if($href != null) $attrs[] = "href=\"{$href}\"";

        $attrs = implode(' ', $attrs);

        echo "<{$tag} class=\"{$classes}\" {$attrs}><span>{$content}</span>{$iconTag}</{$tag}>";
    }

    function ActionButton($content='', $options=null){
        $options = array_merge([
            'class' => 'id3-type-action',
            'style' => 'id3-style-' . $this->settings->style_vars->action_button->style,
            'text_transform' => 'id3-text-' . $this->settings->style_vars->action_button->text_transform,
            'hover_effect'  => 'id3-trans-' . $this->settings->style_vars->action_button->hover_animation,
            'icon_display'  => 'id3-icon-' . $this->settings->style_vars->action_button->icon_display,
            'icon'          => $this->settings->style_vars->action_button->icon,
        ],$options);

        $this->Button($content, $options);
    }
}

global $id3Components;

$id3Components = new Components();