/*
* Gutenberg block building helpers
*/
console.log('ID3BlockEditorBase Declaration');

class ID3BlockEditorBase {

    constructor($params = {shortcode_name:''}){
        this._renderers = null;

        this.title = $params.shortcode_name;
        this.description = '';
        this.category = 'id3';
        this.icon = 'universal-access-alt';
        this.keywords = ['id-3',$params.shortcode_name];
        this.supports = {
            html: true
        }
        this.shortcode_name = $params.shortcode_name;

        // Bind "this" to the edit and save method.
        // This ensure that those method are scoped to the class
        this.edit = this.edit.bind(this);
        this.save = this.save.bind(this);
    }
    

    /**
     * Called by the block editor while editing or rendering the block component
     * @param {*} $props 
     * @returns rendered block in "editing" mode
     */
    edit($props){
        const result = [];
        if(this.configRenderer != undefined && $props.isSelected) result.push(this.configRenderer($props));
        if(this.blockRenderer != undefined) result.push(this.blockRenderer($props));

        if(this._renderers != null){
            result.push(...this._renderers
                .filter($r => {
                    if( $r.mustBeSelected){
                        return $props.isSelected;
                    }
                    return true;
                })
                .map($r => {
                    //$r.preload();
                    return $r.render($props)
                }));
        }
        
        return result;   
    }

    /**
     * Save the block component as a shortcode, passing attributes as shortcode params
     * @param {*} $props 
     * @returns 
     */
    save($props){
        const tag = (this.shortcode_name == '') ? 'id3' : this.shortcode_name;
        const {
            attributes
        } = $props;

        this.validateAttributes($props.attributes);

        let attr = [];
        if(attributes){
            attr = Object.keys(attributes)
                // filter attribute that are set to the default value
                // this prevents error of difference between save and render in Gutenberg, in case a new attribute is added later
                .filter( $k => {    
                    if(this.attributes[$k] == undefined) return false;
                    return attributes[$k] != this.attributes[$k].default;
                })
                // return an array of shortcode params
                .map( $k => {
                    // transform camelCase to underscore_case for param name
                    const attName = $k.replace(/([A-Z])/g, (c) => '_' + c.toLowerCase());
                    return `${attName}="${attributes[$k]}"`;
                });
        }
        
        const result = [
            `[${tag} ${attr.join(' ')}]`
        ];
        
        return result.join('');
    }

    /**
     * Validates and overrides (if needed) attributes
     * @param {*} $attributes 
     */
    validateAttributes($attributes){
        Object.keys($attributes).forEach( $k => {
            $attributes[$k] = this.validateAttribute($k,$attributes[$k]);
        });
    }

    // Must be overwritten
    // Allow data overwrite by validating each key:value of attributes
    validateAttribute($key, $value){
        return $value;
    }

    /**
     * Add a renderer to the list of renderers, which will be use to display block ui, configs or toolbar
     * @param {ID3BlockRenderer} $renderer Renderer object 
     * @param {Boolean} $mustBeSelected Require the isSelected state
     * @returns 
     */
    addRenderer($renderer, $mustBeSelected = false){
        //console.log('addRenderer', $renderer, typeof $renderer, $renderer instanceof ID3BlockRenderer)
        if(! ($renderer instanceof ID3BlockRenderer) ) return;
        if(this._renderers === null) this._renderers = [];

        $renderer.name = this.shortcode_name;
        $renderer.mustBeSelected =  $mustBeSelected;
        this._renderers.push($renderer);
    }
}

class ID3BlockRenderer{

    constructor(){
        this.classList = [];

        this.content = this.content.bind(this);
        this.mustBeSelected = false;
    }

    preload(){

    }

    content(){
        return 'no content';
    }

    containerClassList(){
        const result = [];
        if(this.name != undefined) result.push(this.name.replace(/\_/g,'-'));
        if(this.className != undefined) result.push(this.className);
        if(this.classList.length > 0) result.push(...this.classList);

        return result.join(' ');
    }

    containerStyles(){
        if(this._attributes == undefined) return '';

        const styleVars = Object.keys(this._attributes).reduce( ($result, $k) => {
            const attName = $k.replace(/([A-Z])/g, (c) => '-' + c.toLowerCase());
            const attValue = this._attributes[$k];
            $result['--id3-' + attName] = attValue;
            return $result;
        },{});

        return styleVars;
    }

    render($props){
        const params = $props.attributes;
        this.className = $props.className;
        this.classList = [];
        
        this._attributes = params;

        this.setAttributes = $props.setAttributes;
        this.setAttributes.bind(this);

        return this.content(params);
    }
}
