/**
 * @name LinkLocations
 * @description Transform <a href="maps:..."></a> into a link for google maps when on desktop or android
 */
 class LinkLocations extends ID3AppModule{
    init(){
        const links = id3Query('a[href*=maps]');

        // On Apple device, leave it be
        if(window.navigator.userAgent.indexOf('MacOS') >= 0) return;
        
        links.forEach(link => {
            const url = link.getAttribute('href')
            link.setAttribute('href', url.replace('maps:', 'https://maps.google.com/?daddr='));
            link.setAttribute('target', '_blank');
        });
    }

}
new LinkLocations();

class ID3FrameworkComponents extends ID3AppModule{
    constructor(){
        super('id3-framework');
    }

    init(){
        
        this.addDirective('id3Button',{
            restrict: 'C',
            onApply: ($element) => {
                // apply hover handling
                this.getSettings().then( $settings => {
                    const isStandardStyle = $element.classList.contains('id3-type-normal');
                    const buttonType = (isStandardStyle) ? 'normal' : 'action';
                    const hoverAnimName = 'id3-trans-' + $settings.style_vars[buttonType + '_button'].hover_animation;
                    $element.classList.add('id3-style-' + $settings.style_vars[buttonType + '_button'].style  );
                    $element.classList.add('id3-text-' + $settings.style_vars[buttonType + '_button'].text_transform);
                    $element.classList.add(hoverAnimName);

                });

            }
        })
    }
}
new ID3FrameworkComponents();

class ID3Animate extends ID3AppModule{
    init(){
        this.addDirective(
            'id3Animate', 
            {
                restrict: 'A',
                onApply: function($element){
                
                    $element.addEventListener('animationstart', function($event){
                        $element.classList.add('ali-animate-running');
                    });
        
                    $element.addEventListener('animationend',  function($event){
                        $element.classList.remove('ali-animate-running');
                        $element.classList.add('ali-animate-end');
                        window.setTimeout(_ => {
                            $element.classList.add('ali-animated');
                            $element.classList.remove('ali-animate-end');
                        }, 250);
                    });
                
                    console.log('aliAnimate eventlistener applied', $element);
                }
            }
        );

        this.addDirective(
            'id3HoverClass', 
            {
                restrict: 'A',
                onApply: function($element){
                    const hoverClassAtt = $element.getAttribute('id3HoverClass');
                    if(hoverClassAtt == null) return;
                    const hoverClasses = hoverClassAtt.split( );

                    $element.addEventListener('mouseover', ($event) => {
                        $element.classList.add(...hoverClasses);
                    });

                    $element.addEventListener('mouseout', ($event) => {
                        $element.classList.remove(...hoverClasses);
                    })
                }
            }
        );

        this.addDirective(
            'id3BindToScroll',
            {
                restrict: 'A',
                onApply: function($element){
                    const bodyHeight = document.body.getBoundingClientRect().height;
                    window.addEventListener('scroll', (event) => {
                        if($element._scrollDebounce != undefined){
                            clearTimeout($element._scrollDebounce);
                        }

                        $element._scrollDebounce = setTimeout( () => {
                            const scrollPct = Math.round( (window.scrollY / bodyHeight) * 100);
                            $element.style.setProperty('--id3-scroll-pct', scrollPct + '%');
                            delete $element._scrollDebounce;
                        }, 5);
                    })
                }
            }
        )
    }
}
new ID3Animate();

class ID3Masonry extends ID3AppModule{
    init(){
        
        this.addDirective(
            'id3-style-masonry', 
            {
                restrict: 'C',
                onApply: ($element) => {
                    if($element.querySelector('.id3-list-container')) $element = $element.querySelector('.id3-list-container');
                    this.element = $element;

                    this.onReady($element).then(_ => {
                        
                        this.setResizeObserver($element);
                    });
                }
            }
        );

    }

    onReady($element){
        return new Promise( ($resolve,$reject) => {
            if($element.children.length>0){
                return $resolve();
            }

            this.childObs = new MutationObserver( ($entries) => {
                $resolve();
            });
            this.childObs.observe($element,{childList:true});

        })
    }

    setResizeObserver($element){
        this.resizeObserver = new ResizeObserver( ($entries) => {
            $entries.forEach($entry => {
                this.rowHeight     = parseInt(window.getComputedStyle(this.element).getPropertyValue('grid-auto-rows')) || 20;
                //this.rowGap        = parseInt(window.getComputedStyle(this.element).getPropertyValue('grid-row-gap'));
                console.log('mansonry resize obs', this.rowHeight, this.rowGap);
                this.resizeItem($entry.target);
            })

            //this.resizeObserver.disconnect();
        });
        const childList = Array.from($element.children);
        

        childList.forEach($e => {
            this.resizeObserver.observe($e)
        });
    }

    resizeItem($elm){
        
        const spanDivider   = this.rowHeight;// + this.rowGap;
        const contentElm    = $elm.children[0];
        const elmPadding    = parseInt(window.getComputedStyle(contentElm).getPropertyValue('padding'));
        const itemHeight    = Math.ceil(contentElm.getBoundingClientRect().height) + (elmPadding * 2);//  + this.rowGap;
        const rowSpan       = Math.ceil((itemHeight) / spanDivider);
        $elm.style.gridRowEnd = "span " + rowSpan;

        console.log('resizeItem',elmPadding,itemHeight,rowSpan );
    }
}
new ID3Masonry();
